package org.embulk.input.filename;


import java.util.List;
import java.util.ArrayList;

import com.google.common.base.Optional;

import org.embulk.config.Config;
import org.embulk.config.ConfigDefault;
import org.embulk.config.ConfigDiff;
import org.embulk.config.ConfigSource;
import org.embulk.config.Task;
import org.embulk.config.TaskReport;
import org.embulk.config.TaskSource;
import org.embulk.spi.Exec;
import org.embulk.spi.OutputPlugin;
import org.embulk.spi.PageOutput;
import org.embulk.spi.Schema;
import org.embulk.spi.Page;
import org.embulk.spi.TransactionalPageOutput;
import org.slf4j.Logger;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;


public class JoinfileOutputPlugin
        implements OutputPlugin
{
    public interface PluginTask
            extends Task
    {
        // configuration option 1 (required integer)
        @Config("path_prefix")
        public String getPathPrefix();

        // configuration option 2 (optional string, null is not allowed)
        @Config("file_ext")
        public String getFileExt();
        
        
        @Config("sum_type")
        @ConfigDefault("filename")
        public String getSumType();

    }

    private final Logger log = Exec.getLogger(getClass());
    
    private static FileOutputStream output = null;
    
    private static ArrayList<String> lastP = new ArrayList<String> ();
    
    private static String sumType;
    
    @Override
    public ConfigDiff transaction(ConfigSource config,
            Schema schema, int taskCount,
            OutputPlugin.Control control)
    {
        PluginTask task = config.loadConfig(PluginTask.class);
        
        sumType = task.getSumType();

        // retryable (idempotent) output:
        // return resume(task.dump(), schema, taskCount, control);

        // non-retryable (non-idempotent) output:
        
        String path = task.getPathPrefix() + task.getFileExt();
        
        try {
          output = new FileOutputStream(new File(path));
        } catch (FileNotFoundException ex) {
          throw new RuntimeException (ex);
        }
        
        
        // for the ConfigDiff, we set the last Path of each task is "" as default.
        for (int i = 0 ; i< taskCount; i++)
        {
            lastP.add("");
        }
        
        
        control.run(task.dump());
        
        
        closeFile();
        return Exec.newConfigDiff();
    }

    @Override
    public ConfigDiff resume(TaskSource taskSource,
            Schema schema, int taskCount,
            OutputPlugin.Control control)
    {
        throw new UnsupportedOperationException("joinfile output plugin does not support resuming");
    }

    @Override
    public void cleanup(TaskSource taskSource,
            Schema schema, int taskCount,
            List<TaskReport> successTaskReports)
    {
    }

    @Override
    public TransactionalPageOutput open(TaskSource taskSource, Schema schema, int taskIndex)
    {
        PluginTask task = taskSource.loadTask(PluginTask.class);
        
        final int ind = taskIndex;
        
        return new TransactionalPageOutput(){
          //private final List<String> filenames = new ArrayList<>() ;
          
          public void add(Page page){
            //log.info("The ADD: " + page.getStringReferences() + " ## " +page.getValueReferences());
            try {
                    List<String> pageArray = page.getStringReferences();
                    String content = page.getStringReference(0);
                    String line = page.getStringReference(1) + "\n";
                    String tag = page.getStringReference(1);
                    if (sumType.equals("filename")){
                        output.write(line.getBytes());
                    }else{
                        output.write(content.getBytes());
                    }
                    lastP.set(ind ,tag);
                
            } catch (IOException ex) {
                
                throw new RuntimeException(ex);
            }
          }
          
          public void finish(){
            //log.info("Finished");
          }
          
          public void close(){
            //log.info("closed");
          }
          
          public void abort(){
            
          }
          
          public TaskReport commit(){
            return Exec.newTaskReport();
            
          }
          
        };
        // Write your code here :)
        //throw new UnsupportedOperationException("JoinfileOutputPlugin.run method is not implemented yet");
    }
    
    public static void closeFile()
    {
      if (output!= null){
        try {
          output.close();
        }catch (IOException ex ) {
          throw new RuntimeException(ex);
        }
      }
    }
}
